; Kernel Include File

; Macros
rkcall .equ 0x08
rlcall .equ 0x10

.macro kcall(addr)
    rst rkcall
    call addr
.endmacro
.macro kcall(cc, addr)
    rst rkcall
    call cc, addr
.endmacro
.macro kjp(addr)
    rst rkcall
    jp addr
.endmacro
.macro kjp(cc, addr)
    rst rkcall
    jp cc, addr
.endmacro
.macro kld(to, from)
    rst rkcall
    ld to, from
.endmacro

.macro lcall(id, addr)
    rst rlcall
    .db id
    call addr
.endmacro
.macro lcall(id, cc, addr)
    rst rlcall
    .db id
    call cc, addr
.endmacro
.macro ljp(id, addr)
    rst rlcall
    .db id
    jp addr
.endmacro
.macro ljp(id, cc, addr)
    rst rlcall
    .db id
    jp cc, addr
.endmacro
.macro lld(id, to, from)
    rst rlcall
    .db id
    ld to, from
.endmacro

.macro icall(addr)
    rst rlcall
    .db libId
    call addr
.endmacro
.macro icall(cc, addr)
    rst rlcall
    .db libId
    call cc, addr
.endmacro
.macro ijp(addr)
    rst rlcall
    .db libId
    jp addr
.endmacro
.macro ijp(cc, addr)
    rst rlcall
    .db libId
    jp cc, addr
.endmacro
.macro ild(to, from)
    rst rlcall
    .db libId
    ld to, from
.endmacro
; /Macros

; Errors
errOutOfMem         .equ 1
errTooManyThreads   .equ 2
errStreamNotFound   .equ 3
errEndOfStream      .equ 4
errFileNotFound     .equ 5
errTooManyStreams   .equ 6
errNoSuchThread     .equ 7
errTooManyLibraries .equ 8
errUnsupported      .equ 9
errTooManySignals   .equ 10

; Keypad codes
kDown               .equ 0x01
kLeft               .equ 0x02
kRight              .equ 0x03
kUp                 .equ 0x04
kEnter              .equ 0x09 \kEntry .equ 0x09 \kSolve .equ 0x09
kPlus               .equ 0x0A \kMem .equ 0x0A \kQuote .equ 0x0A
kMinus              .equ 0x0B \kLSqBracket .equ 0x0B \kW .equ 0x0B
kMul                .equ 0x0C \kRSqBracket .equ 0x0C \kR .equ 0x0C
kDiv                .equ 0x0D \kVarE .equ 0x0D \kM .equ 0x0D
kPower              .equ 0x0E \kPi .equ 0x0E \kH .equ 0x0E
kClear              .equ 0x0F
kChs                .equ 0x11 \kNeg .equ 0x11 \kAns .equ 0x11 \kQuestion .equ 0x11
k3                  .equ 0x12 \kTheta .equ 0x12 \kL3 .equ 0x12
k6                  .equ 0x13 \kL6 .equ 0x13 \kV .equ 0x13
k9                  .equ 0x14 \kVarW .equ 0x14 \kQ .equ 0x14
kRParen             .equ 0x15 \kRBrace .equ 0x15 \kL .equ 0x15
kTan                .equ 0x16 \kInvTan .equ 0x16 \kG .equ 0x16
kVars               .equ 0x17 \kDistr .equ 0x17
kDecPnt             .equ 0x19 \kDot .equ 0x19 \kPeriod .equ 0x19 \kVarI .equ 0x19 \kImaginary .equ 0x19 \kColon .equ 0x19
k2                  .equ 0x1A \kL2 .equ 0x1A \kZ .equ 0x1A
k5                  .equ 0x1B \kL5 .equ 0x1B \kU .equ 0x1B
k8                  .equ 0x1C \kVarV .equ 0x1C \kP .equ 0x1C
kLParen             .equ 0x1D \kLBrace .equ 0x1D \kK .equ 0x1D
kCos                .equ 0x1E \kInvCos .equ 0x1E \kF .equ 0x1E
kPrgm               .equ 0x1F \kDraw .equ 0x1F \kC .equ 0x1F
kStat               .equ 0x20 \kList .equ 0x20
k0                  .equ 0x21 \kCatalog .equ 0x21 \kSpace .equ 0x21
k1                  .equ 0x22 \kL1 .equ 0x22 \kY .equ 0x22
k4                  .equ 0x23 \kL4 .equ 0x23 \kT .equ 0x23
k7                  .equ 0x24 \kVarU .equ 0x24 \kO .equ 0x24
kComma              .equ 0x25 \kEE .equ 0x25 \kJ .equ 0x25
kSin                .equ 0x26 \kInvSin .equ 0x26 \kE .equ 0x26
kApps	            .equ 0x27 \kAngle .equ 0x27 \kB .equ 0x27
kGraphvar           .equ 0x28 \kXTThetaN .equ 0x28 \kLink .equ 0x28
kStore              .equ 0x2A \kRcl .equ 0x2A \kX .equ 0x2A
kLn                 .equ 0x2B \kEx .equ 0x2B \kS .equ 0x2B
kLog                .equ 0x2C \k10x .equ 0x2C \kN .equ 0x2C
kSquare             .equ 0x2D \kXSqu .equ 0x2D \kSqRoot .equ 0x2D \kI .equ 0x2D
kRecip              .equ 0x2E \kXNeg .equ 0x2E \kMatrix .equ 0x2E \kD .equ 0x2E
kMath               .equ 0x2F \kA .equ 0x2F \kTest .equ 0x2F
kAlpha              .equ 0x30 \kALock .equ 0x30
kGraph              .equ 0x31 \kTable .equ 0x31 \kF5 .equ 0x31
kTrace              .equ 0x32 \kCalc .equ 0x32 \kF4 .equ 0x32
kZoom               .equ 0x33 \kFormat .equ 0x33 \kF3 .equ 0x33
kWindow             .equ 0x34 \kTblSet .equ 0x34 \kTableSet .equ 0x34 \kF2 .equ 0x34
kYEqu               .equ 0x35 \kStatPlot .equ 0x35 \kF1 .equ 0x35
k2nd                .equ 0x36
kMode               .equ 0x37 \kQuite .equ 0x37
kDel                .equ 0x38 \kIns .equ 0x38 \kDelete .equ 0x38

; The rest of this file was generated by a tool.
.equ BOOT 0x0059
.equ REBOOT 0x0078
.equ SUSPENDDEVICE 0x300E
.equ SYSINTERRUPT 0x01C3
.equ MALLOC 0x04E3
.equ FREE 0x0587
.equ CALLOC 0x046D
.equ MEMSET 0x0486
.equ FORMATMEM 0x044A
.equ MEMSEEKTOSTART 0x04A5
.equ MEMSEEKTOEND 0x04CE
.equ LOADLIBRARY 0x10BA
.equ CONTEXTSWITCH 0x01B2
.equ STARTTHREAD 0x0D2A
.equ LAUNCHPROGRAM 0x0EBA
.equ GETCURRENTTHREADID 0x0D0D
.equ EXITTHREAD 0x0F1C
.equ KILLTHREAD 0x0E22
.equ KILLCURRENTTHREAD 0x0DB6
.equ GETTHREADENTRY 0x0F43
.equ SETRETURNPOINT 0x0F5C
.equ SUSPENDCURRENTTHREAD 0x1099
.equ RESUMETHREAD 0x10AC
.equ SETINITIALHL 0x0FE4
.equ SETINITIALDE 0x0FB6
.equ CREATESIGNAL 0x11B9
.equ READSIGNAL 0x11F8
.equ WAITKEY 0x2EF8
.equ GETKEY 0x2F1B
.equ FLUSHKEYS 0x2F04
.equ FASTCOPY 0x1C41
.equ LCDDELAY 0x308F
.equ GETBATTERYLEVEL 0x30C2
.equ GETLCDLOCK 0x125C
.equ GETKEYPADLOCK 0x126E
.equ GETIOLOCK 0x1265
.equ GETUSBLOCK 0x1277
.equ HASLCDLOCK 0x1280
.equ HASKEYPADLOCK 0x129A
.equ HASIOLOCK 0x128D
.equ HASUSBLOCK 0x12A7
.equ GETTIMEINTICKS 0x2FF4
.equ SETCLOCK 0x2FDC
.equ GETTIME 0x3006
.equ CONVERTTIMEFROMTICKS 0x3004
.equ CONVERTTIMETOTICKS 0x3005
.equ UNLOCKFLASH 0x0602
.equ LOCKFLASH 0x0618
.equ COPYFLASHPAGE 0x07E7
.equ COPYSECTORTOSWAP 0x0751
.equ ERASEFLASHSECTOR 0x06CE
.equ ERASEFLASHPAGE 0x072A
.equ WRITEFLASHBUFFER 0x0677
.equ WRITEFLASHBYTE 0x062E
.equ ERASESWAPSECTOR 0x06C8
.equ DELETEFILE 0x0868
.equ FILEEXISTS 0x0893
.equ FINDFILEENTRY 0x08A5
.equ OPENFILEREAD 0x09A6
.equ GETSTREAMENTRY 0x0A28
.equ CLOSESTREAM 0x0A4C
.equ STREAMREADBYTE 0x0A64
.equ STREAMREADWORD 0x0B27
.equ STREAMREADBUFFER 0x0B39
.equ STREAMREADTOEND 0x0CE9
.equ GETSTREAMINFO 0x0C30
.equ ALLOCSCREENBUFFER 0x045C
.equ CLEARBUFFER 0x12B5
.equ GETPIXEL 0x12C9
.equ SETPIXEL 0x12EC
.equ RESETPIXEL 0x12F6
.equ INVERTPIXEL 0x1301
.equ DRAWLINE 0x130B
.equ PUTSPRITEOR 0x1613
.equ PUTSPRITEAND 0x155B
.equ PUTSPRITEXOR 0x14A6
.equ RECTXOR 0x16C8
.equ RECTOR 0x1730
.equ RECTAND 0x1798
.equ PUTSPRITE16XOR 0x1801
.equ PUTSPRITE16OR 0x1874
.equ PUTSPRITE16AND 0x18E7
.equ DRAWCHAR 0x28A2
.equ DRAWCHARAND 0x28D0
.equ DRAWCHARXOR 0x28FE
.equ DRAWSTR 0x292C
.equ DRAWSTRAND 0x293B
.equ DRAWSTRXOR 0x294A
.equ DRAWSTRFROMSTREAM 0x295A
.equ DRAWHEXA 0x296A
.equ MEASURECHAR 0x2991
.equ MEASURESTR 0x29A4
.equ DRAWHEXHL 0x2986
.equ CRC16 0x3300
.equ SHA1INIT 0x3324
.equ SHA1CLEAN 0x336F
.equ SHA1ADDBYTE 0x33B2
.equ SHA1ADDRANGE 0x3595
.equ SHA1PAD 0x3385
.equ GETBOOTCODEVERSIONSTRING 0x31AF
.equ CPHLDE 0x3097
.equ CPHLBC 0x309C
.equ CPBCDE 0x30A1
.equ DEMULA 0x30E4
.equ DIV32BY16 0x3143
.equ SUB16FROM32 0x3161
.equ DIVHLBYC 0x318F
.equ DIVACBYDE 0x319C
.equ STRINGLENGTH 0x30A9
.equ STRINGCOPY 0x3103
.equ COMPARESTRINGS 0x30F6
.equ SORT 0x3112
.equ HEXTOA 0x3064
.equ HEXTOHL 0x3031
.equ RLECOMPRESS 0x31D7
.equ RLEDECOMPRESS 0x328E
.equ RLECALCULATECOMPRESSEDLENGTH 0x3239
.equ RLECALCULATEDECOMPRESSEDLENGTH 0x32C2
.equ RANDA 0x32E6
; Describes kernel memory layout

; Modify these values to automatically modify memory layout
maxThreads              .equ 32
maxLibraries            .equ 16
maxSignals              .equ 16
maxFileStreams          .equ 16
kernelGarbageSize       .equ 0x200
stateMemorySize         .equ 0x100

kernelMem .equ 0x8000

threadTable             .equ kernelMem
threadTableSize         .equ 8 * maxThreads             ; 8 bytes per entry

libraryTable            .equ threadTable + threadTableSize
libraryTableSize        .equ 4 * maxLibraries           ; 4 bytes per entry

signalTable             .equ libraryTable + libraryTableSize
signalTableSize         .equ 4 * maxSignals             ; 4 bytes per entry

fileHandleTable         .equ signalTable + signalTableSize
fileHandleTableSize     .equ 8 * maxFileStreams         ; 8 bytes per entry

kernelTablesSize        .equ threadTableSize + libraryTableSize + signalTableSize + fileHandleTableSize

stateMemory             .equ kernelMem + kernelTablesSize

currentThreadIndex      .equ stateMemory                ; 1 byte
activeThreads           .equ currentThreadIndex + 1     ; 1 byte
loadedLibraries         .equ activeThreads + 1          ; 1 byte
activeSignals           .equ loadedLibraries + 1        ; 1 byte
activeFileStreams       .equ activeSignals + 1          ; 1 byte
lastThreadId            .equ activeFileStreams + 1      ; 1 byte

hwLockIO                .equ lastThreadId + 1           ; 1 byte
hwLockLCD               .equ hwLockIO + 1               ; 1 byte
hwLockKeypad            .equ hwLockLCD + 1              ; 1 byte
hwLockUSB               .equ hwLockKeypad + 1           ; 1 byte

; Misc.
currentContrast         .equ hwLockUSB + 1              ; 1 byte
bcallHook               .equ currentContrast + 1        ; 2 bytes

; Points to the font table used by the kernel.
; To reset, write kernel_font to it.
fontTablePtr            .equ currentContrast + 2        ; 2 bytes

; Used for line drawing
; TODO: Phase this out
clip_mask               .equ fontTablePtr + 2           ; 1 byte

kernelGarbage           .equ stateMemory + stateMemorySize

userMemory              .equ kernelGarbage + kernelGarbageSize
userMemorySize          .equ 0xFFFF - userMemory        ; user memory extends to 0xFFFF
